function c = AdamsBashforth(a)
%
%  determines the coefficients in the Adams-type formulas 
%  used to solve  y'(x)=f(x,y).
%
%  The form of these integration formulas is 
%
%  y_{n+1} = y_{n} + h*\sum(cof(j)*f(j))
%
%  where f(j) denotes the value of f for
%
%  x(j) = x_{n} + h*a(j)
%
%   The weighting coefficient cof(j) equals the quotient c(j,1)/c(j,2).
%
%   AdamsBashforth(0:-1:-4)  determines the Adams-Bashforth formula of order 5.
%   AdamsBashforth(1:-1:-3)  determines the Adams-Bashforth formula of order 5.
%
%  a vector of coefficients defining the base points used for the integration
%  c     weighting coefficients for the integration formula
%

a           = a(:)'; 
n           = length(a);
a           = (ones(n,1)*a).^((0:n-1)'*ones(1,n));
[ctop,cbot] = rat(a\(1 ./(1:n)'));
c           = [ctop,cbot];

