%   Gauss-Legendre quadrature in one dimension 
%
%  Integrate f(x) = exp(-x^2/2)     on [-1,1]
%  Integrate f(x) = 1+x^2-3x^3+4x^5 on [-1,1]
%
%  parameters: 
%
%  x = nodes
%  w = weights
%  n = number of integration points
%
%  Uses NodesWeights.m
%

m = 5;

fprintf('\n\n');
disp(['GaussLegendre quadrature']);
fprintf('\n');

fprintf('    n         result \n');

result = zeros(m,1);

for n=1:m
  [x,w] = NodesWeights(n);

%  integrate	

  result(n) = 0.0;

  for i=1:n
    X      = x(i);
    W      = w(i);
    f      = 1+X^2-3*X^3+4*X^5; % compute function value at the nodes
    f      = exp(-X^2/2);  	% 
    result(n) = result(n)+f*W;
  end

end

disp([(1:n)',result(1:n)]);        % print the solution


