function extrpol(x)
% returns Richardson Extrapolation table for a given x and f(x). 
%
%  algorithm to compute a table of Richardson Extrapolations
%    for Computing the derivative
%
%  given a function f(x), which is defined in f.m;
% 
%    x     = x-value (input)             
%    h     = h-value to start         
%    maxst = maximum number of stages
%    tol   = tolerance value for termination
%
%  Gerald-Wheatley pg. 326-327.
%
%  USAGE: [d]=extrpol(x)
%
%  EXAMPLE
%  maxst=10; x=1.0; h0=0.1; tol=10^-7;
%>> extrpol(x)
% Richardson Table
%   0.2267
%   0.2360    0.2391
%   0.2384    0.2391    0.2391
%   0.2389    0.2391    0.2391    0.2391
%
      if (nargin==0),
        help extrpol; return
      end

      disp(['Richardson Table']);

      maxst = 10 ;
      h0    = 0.1;
      tol   = 1.0e-7;
%
      d = zeros(maxst+1,maxst+1);

      h = h0;
      d(1,2) = 0.0;
%
%  Compute lines of table
%
      for st=0:maxst;
        d(st+1,1) = (f(x+h)-f(x-h))/(2*h);

        for j=1:st;
          d(st+1,j+1) = d(st+1,j) + (d(st+1,j)-d(st,j))/((2^(2*j))-1);
        end;
      
       if ~(st == 0) 
       if (abs(d(st+1,st+1)-d(st+1,st)) < tol)
          for i=0:st;
            disp([d(i+1,1:i+1)]);
          end;
          return;
        end;
       end;
  
        h = h/2;
        
      end;
% 
