function y=trapsimp(x)
% QUAD   Numerically evaluate integral, adaptive Simpson quadrature.
%    Q = QUAD(FUN,A,B) tries to approximate the integral of function
%    FUN from A to B to within an error of 1.e-6 using recursive
%    adaptive Simpson quadrature.  The function Y = FUN(X) should
%    accept a vector argument X and return a vector result Y, the
%    integrand evaluated at each element of X.  
%
%
%       FUN can be specified as:
% 
%        An inline object:
%           F = inline('(4-(x.^2)).^.5');
%
%  example 1
%
%>> F=inline('(4-(x.^2)).^.5');
%>> quad(F,0,2)
%>> 3.146
%
%  example 2
%>> quad('trapsimp',0,2)
%
%ans =
%
%    3.1416

y=(4-(x.^2)).^.5;

