      program GaussLegendre 

      use gauleg_module, only : real8, gauleg

      implicit none

      real(kind=real8)                 :: x1
      real(kind=real8)                 :: x2
      real(kind=real8), dimension(100) :: x
      real(kind=real8), dimension(100) :: w
      real(kind=real8)                 :: eps
      integer                          ::  n,i,k
!
!***********************************************************************
!  this program generates weights given a number (n) of abscissae on the
!  interval (x1,x2). the code has been restricted to n \in {1,2,3,4,5} and 
!  x1 = 0.0 x2 = 1.0
!
!  based on a (more general) program by hermann von ulrich 23. september 1987
!     converted f90, 200409, m. r. schaferkotter. schaferk@bellsouth.net
!  
!  the code can easily be modified to accept interval endpoints other than {-1,1}.
!***********************************************************************
! 
!
!***********************************************************************
!  to build the program:
!
!  f90 -c gauleg.f90
!  f90 -c GaussLegendre.f90
!  f90 -o GaussLegendre.x  GaussLegendre.o gauleg.o 
!***********************************************************************
!

      print *
      print *
      print *,"GaussLegendre quadrature nodes and weights"
      print *
!
!     print *," input the level of precision, 3.0d-17 will give you roughly 16 digits, i.e. double precision."
!     read(unit=*,fmt=*) eps
!
!***********************************************************************
!  setting precision 
!***********************************************************************
!
      eps = 3.0d-17
!
!      print *," input lower and then upper bound for the gauss-legendre quadrature."
!      read (unit=*,fmt=*) x1,x2
!
!***********************************************************************
!  setting interval to (-1.0,1.0)
!***********************************************************************
!
      x1 = -1.0
      x2 =  1.0
!
      print *," input the number of weights (1 <= n <=5 )."
      read(unit=*,fmt=*) n
!
      select case (n)
        case(5)
          x(1:5) = (/ -0.906179845938664, -0.538469310105683, 0.0, 0.538469310105683, 0.906179845938664 /)
        case(4)
          x(1:4) = (/ -0.861136311594053, -0.339981043584856, 0.339981043584856, 0.861136311594053 /)
        case(3)
          x(1:3) = (/ -0.774696669241483, 0.0, 0.774696669241483 /)
        case(2)
          x(1:2) = (/ -0.577350269189626, 0.577350269189626 /)
        case(1) 
          x(1:1) = (/ 0.0 /)
        case default
          print *,"improper case for n=",n
          print *,"program may be modified for the case." 
          stop
      endselect
 
      call gauleg(x1,x2,x,w,n,eps)

      print *,"   m    x                  w"
      print *,"----------------------------------------------"

      if(modulo(n,2)==0)then
        k = 0
        do i = 1,(n/2)
          k = k+1
          print "(i5,tr2,2f20.16)", k,-x(n/2-i+1),w(n/2-i+1)
        enddo
        do i = n,(n/2)+1,-1
           k = k+1
           print "(i5,tr2,2f20.16)", k,x(n-i+1),w(n-i+1)
        enddo
      endif

      if(modulo(n,2)==1)then 
        k = 0
        do i = 1,(n/2)+1
          k = k+1
          print "(i5,tr2,2f20.16)", k,-x(n/2-i+1),w(n/2-i+1)
        enddo
        do i = n,(n/2)+2,-1
          k = k+1 
          print "(i5,tr2,2f20.16)", k,x(n-i+1),w(n-i+1)
        enddo
      endif

      print *
      print *

      endprogram GaussLegendre

