      program difftabl
      use binocoef_module, only : binocoef
      implicit none
!
!***********************************************************************
!  difftabl - generates and applies differences table for
!  interpolation and differentiation. 
!***********************************************************************
!
      integer                            :: bc 
      integer                            :: i
      integer                            :: ir 
      integer                            :: ic
      integer                            :: io
      integer                            :: k 
      integer                            :: n 
      integer                            :: r 
      integer                            :: sgn 

      real                               :: dx
      real                               :: fx 
      real                               :: xg 
      real            , dimension(4)     :: c
      real            , dimension(100)   :: x
      real            , dimension(100,5) :: yanddy

      character(len=1)                   ::  ans,ms
!
      print *
      print *
      print *,"difftabl"
      print *

      print "(a,/,tr19,a)", " generate difference table up to fourth-order difference"
!
      print *, "input number of data points, n (>4) : "
      read  (unit=*,fmt=*) n
      print *, " input x values and press <enter> key after entering each number :"
!
      do i=1,n
        read  (unit=*,fmt=*) x(i)
      enddo
!
      print *, " input y values and press <enter> key after entering each number :"
!
      do i=1,n
        read  (unit=*,fmt=*) yanddy(i,1)
      enddo
!
!***********************************************************************
!     calculate finite differences
!***********************************************************************
!
      do ic=2,5
        do ir=1,n-ic+1
          yanddy(ir,ic)=yanddy(ir+1,ic-1)-yanddy(ir,ic-1)
        enddo
      enddo
!
      dx = x(2)-x(1)
!
      print *, " i      xi        yi       dyi       d2yi      d3yi      d4yi"

      print "(i2,2f10.5,2f20.5)" , 1,x(1),yanddy(1,1)
      print "(tr12,2f20.5)"      , yanddy(1,2)
      print "(i2,2f10.5,2f20.5)" , 2,x(2),yanddy(2,1),yanddy(1,3)
      print "(tr12,2f20.5)"      , yanddy(2,2),yanddy(1,4)
!
      do i=3,n-2
        print "(i2,2f10.5,2f20.5)", i,x(i),yanddy(i,1),yanddy(i-1,3),yanddy(i-2,5)
        print "(tr12,2f20.5)"     , yanddy(i,2),yanddy(i-1,4)
      enddo
!
      print "(i2,2f10.5,2f20.5)"  , n-1,x(n-1),yanddy(n-1,1),yanddy(n-2,3)
      print "(tr12,2f20.5)"       , yanddy(n-1,2)
      print "(i2,2f10.5,2f20.5)"  , n,x(n),yanddy(n,1)
!
   37 write (unit=*,fmt="(/,' (i)nterpolate (d) find derivative of y(x) for given x value ,(q)uit?','  : ')")
   !37 write (unit=*,fmt="(/,' (i)nterpolate (d) find derivative of y(x) for given x value,',/,'(q)uit?','  : ')")
!
      read  (unit=*,fmt="(a1)") ans
!
      if (ans == 'q') then
        stop 
      endif
!
      print "(' input x value where interpolation or derivative is to be calculated : ')"
      read  (unit=*,fmt=*) xg
!
      print "(' (f)orward or (b)ackward difference? :')"
!
      read (unit=*,fmt="(a1)") ms
!
      do 125 k=1,n
!
        if (xg > x(k)) then
          goto 125
        endif
!
        i=k-1
!
        if (i > 0) then
          goto 210
        endif
!
        i=1
!
        goto 150
  125 continue
!
      i=n-1
!
  150 write (unit=*,fmt=*) '** extrapolation ! **'
!
  210 if (ms == 'f') then
        goto 220
      endif
!
      r=(x(i+1)-xg)/(x(i+1)-x(i))
!
      goto 230
!
  220 r=(xg-x(i))/(x(i+1)-x(i))
!
  230 write (unit=*,fmt="(' input the highest order of finite differences required :')")
!
      read  (unit=*,fmt=*) io
!
      if (ans == 'i') then
        goto 285
      endif
!
      if (ans /= 'd') then
        stop 
      endif
!
      if ((r /= 1.0).and.(r /= 0.0)) then
        goto 234
      endif
!
      i=i+1
!
      goto 237
!
  234 c(1) = 1
      c(2) = r-0.5
      c(3) = r**2/2-r+1.0/3
      c(4) = r**3/6-0.75*r**2+11*r/12-0.25
!
      if (ms == 'b') then
        goto 255
      endif
!
!***********************************************************************
!  differentiation using forward differences (1) with interpolation
!***********************************************************************
!
      fx = 0
!
      do k=1,io
        fx=fx+c(k)*yanddy(i,k+1)
      enddo
!
      goto 265
!
  237 if (ms == 'b') then
        goto 245 
      endif
!
!***********************************************************************
!       (2) without interpolation                    
!***********************************************************************
!
      fx=0
!
      do k=1,io                        
!
        if (i > n-k) then
          write (unit=*,fmt="(/,' improper request (order too high); reenter input data!')")
          stop
        endif
!
        sgn = (-1)**(k+1)
        fx  = fx+sgn*yanddy(i,k+1)/k
      enddo
!
      goto 265
!
!***********************************************************************
!     differentiation using backward differences (1) without interpolation
!***********************************************************************
!
  245 fx=0
!
      do k=1,io
!
        if ((i-k) < 1) then
          write (unit=*,fmt="(/,' improper request (order too high); reenter input data!')")
          stop
        endif
!
        fx=fx+yanddy(i-k,k+1)/k
      enddo
!
      goto 265
!
!***********************************************************************
!       (2) with interpolation
!***********************************************************************
!
  255 fx=0
!
      do k=1,io
!
        if ((i+1-k) < 1) then
          write (unit=*,fmt="(/,' improper request (order too high); reenter input data!')")
          stop
        endif
!
        fx=fx+(-1)**(k+1)*c(k)*yanddy(i+1-k,k+1)
!
      enddo
!
  265 fx=fx/dx
!
      write (unit=*,fmt="(/,' the calculated dy/dx = ',e12.5)") fx
      goto 37
!
  285 if (ms == 'b') then
        goto 335
      endif
!
!***********************************************************************
!     interpolation using forward differences
!***********************************************************************
!
      fx=yanddy(i,1)
!
      do k=1,io
!
        if (i > (n-k)) then
          write (unit=*,fmt="(/,' improper request (order too high); reenter input data!')")
          stop
        endif
!
        call binocoef(r,k,bc)
!
        fx=fx+bc*yanddy(i,k+1)
      enddo
!
      goto 400
!
!***********************************************************************
!     interpolation using backward differences
!***********************************************************************
!
  335 i=i+1
!
      if (i > n) then
        i=n
      endif
!
      fx=yanddy(i,1)
!
      do k=1,io
!
        if ((i-k) < 1) then
          write (unit=*,fmt="(/,' improper request (order too high); reenter input data!')")
          stop
        endif
!
        call binocoef(r,k,bc)
!
        fx=fx+(-1)**k*bc*yanddy(i-k,k+1)
      enddo
!
  400 write (unit=*,fmt="(/,' answer is : y(x) = ',e12.5)") fx
      goto 37
!
      endprogram difftabl
!
