      program extrpol

      use f_module, only : f

      implicit none
!
!***********************************************************************
!  algorithm to compute a table of Richardson Extrapolations 
!    for computing the derivative 
!
!  given a function f(x), which is defined in module f.f90;

!  inputs:
!    x     = x-value
!    h     = h-value to start
!    maxst = maximum number of stages
!    tol   = tolerance value for termination
!
!  reference: Gerald-Wheatley pg. 326-327.
!***********************************************************************
!
      integer, parameter                  :: maxst = 10 
!
      real   , parameter                  :: x     = 1.0 
      real   , parameter                  :: h0    = 0.1
      real   , parameter                  :: tol   = 1.0e-7
!
      real                                :: h
      real   , dimension(0:maxst,0:maxst) :: d

      integer                             :: st
      integer                             :: i 
      integer                             :: j
!
 
      print *
      print *
     
      print *,"maxst =", maxst
      print *,"x     =", x
      print *,"h0    =", h0
      print *,"tol   =", tol
      print *
      print *, "Richardson Table"
!
      h      = h0
      d(0,1) = 0.0
!
!***********************************************************************
!  compute lines of table
!***********************************************************************
!
      do st=0,maxst

        d(st,0) = (f(x+h)-f(x-h))/(2*h)

        do j=1,st
          d(st,j) = d(st,j-1) + (d(st,j-1)-d(st-1,j-1))/((2**(2*j))-1)
        enddo

        if(abs(d(st,st)-d(st,st-1)) < tol) then
!
!***********************************************************************
!  print table on convergence
!***********************************************************************
!
          do i=0,st
            print *,d(i,0:i)
          enddo
          exit
        endif

        h = h/2
      enddo

      print *
!
      endprogram extrpol
! 
