! %w% latest revision %g% %u%
      program ffttest
!
      use fft_module, only : rl, fft
!
      implicit none
!
      integer      , parameter          :: mp  = 2      !n     = 2**mp
      integer      , parameter          :: n   = 2**mp  !transform length
      integer      , parameter          :: isn = 3      !scale = 1/sqrt(n); see fft.f90 
!
      complex      , dimension(n)       :: a 
      complex      , dimension(n)       :: a0
      real(kind=rl), dimension(n)       :: fr
      real(kind=rl), dimension(n)       :: fi 
!
      integer                           :: j
      real                              :: coeff
      complex                           :: pi2
      complex                           :: ci   
!
!
      print *
      print *
      print *,"fft"
      print *

      pi2 = cmplx(2.0*3.1415927,0.0)
      ci  = cmplx(0.0          ,1.0)
!
!***********************************************************************
!  fill a with initial data
!***********************************************************************
!
      do  j=1,n
        coeff = real(-1*(j-1))/real(n)
        a(j)  = exp(pi2*ci*cmplx(coeff,0.0))
      enddo
!
      a0 = a 
!
      fr = real (a)
      fi = aimag(a)
!
!***********************************************************************
!  display array as complex array.
!***********************************************************************
!
      print *
      print *
      print "(tr26,'input data matrix a')"
!
      do j=1,n
        print "(i5,tr2,a,f6.3,a,f6.3,a)",j,"(",fr(j),",",fi(j),")"
      enddo
!
!***********************************************************************
!  transform 
!***********************************************************************
!
      call fft(fr,fi,mp=mp,isw=isn)
      print "(/,tr26,'fft a')"
      do j=1,n
        print "(i5,tr2,a,f6.3,a,f6.3,a)",j,"(",fr(j),",",fi(j),")"
      enddo
!
!***********************************************************************
!  inverse transform
!***********************************************************************
!
      call fft(fr,fi,mp=mp,isw=-isn)
      print "(/,tr26,'inverse fft (fft a)')"
      do j=1,n
        print "(i5,tr2,a,f6.3,a,f6.3,a)",j,"(",fr(j),",",fi(j),")"
      enddo
!
!***********************************************************************
!  compare result with original
!***********************************************************************
!
      print "(/,tr26,'a-fft^-1(fft(a)) ')"
      do j=1,n
        print "(i5,tr2,a,f6.3,a,f6.3,a)",j,"(",real(a0(j))-fr(j),",",aimag(a0(j))-fi(j),")"
      enddo

      print *
      print *
!
!***********************************************************************
!  example for n = 4  (n=2**mp)
!
!                        the input matrix is below
!  1  ( 1.000, 0.000)  
!  2  ( 0.000,-1.000)  
!  3  (-1.000, 0.000)  
!  4  ( 0.000, 1.000)
!
!                       the output matrix is below
!  1  (  0.00,  0.00)  
!  2  (  2.00,  0.00)  
!  3  (  0.00,  0.00)  
!  4  (  0.00,  0.00)
! 
       endprogram ffttest
!
