module ht_module

implicit none

public :: ht

integer, private, parameter :: real8 = selected_real_kind(10)

intrinsic modulo, sqrt, real

private :: casx

!
!*******************************************************************************
!  this subroutine is used by the program ht2d.f90
!*******************************************************************************
!

contains
 
      subroutine ht(n,x,y)
      integer, intent(in)                      :: n
      real   , intent(in)   , dimension(0:n-1) :: x 
      real   , intent(inout), dimension(0:n-1) :: y 
!
!*******************************************************************************
! ht computes a hartley transform.    
!
!    the discrete hartley transform (dht) y of a data set x is
!
!      y(i) = 1/\sqrt(n)*\sum_{0\le j\le n-1} x(j)*cas(2*pi*i*j/n).
!
!    the data is normalized with a factor of 1/sqrt(n), which makes the transform is involutional:
!    i.e., an operator  which satisfies f^-1(f^-1(x))=x.
!    the transform is its own inverse.
!*******************************************************************************
!
      integer          :: i
      integer          :: j
      real             :: t
      real(kind=real8) :: pi
      real(kind=real8) :: rn
!
      pi   = 2.0_real8*asin(1.0_real8)
      rn   = 1.0_real8/sqrt(real(n,kind=real8)) 
!
      y(0:n-1) = 0.0_real8

      do j = 0, n-1
        do i = 0, n-1
          t    = 2.0_real8*pi*real(modulo(i*j,n))/real(n,kind=real8)
          y(i) = y(i) + x(j)*casx(t)
        end do
      end do

      y(0:n-1) = rn*y(0:n-1)

   endsubroutine ht 

      function casx (x) result(casx_result)
      real, intent(in) :: x
      real             :: casx_result
!
!*******************************************************************************
!    the "casine" (discrete hartley transform) is written cas(x) and is defined by:
!      cas(x) = cos(x) + sin(x)
!***********************************************************************
!
  casx_result = cos(x)+sin(x)

   endfunction casx

end module ht_module

