      program trapsimp
!
!***********************************************************************
! numerical integration using trapezoidal or simpson's rule
!      define f=sqrt(4.0-(x)**2) below
!***********************************************************************
!
      real   , dimension(:), allocatable :: y
!
      integer                            :: i,k,ios
!
      integer                            :: n      ! number of integration points
      real                               :: a      ! lower limit of integration
      real                               :: b      ! upper limit of integration

      real                               :: smyie 
      real                               :: smyio 
      real                               :: ans
      real                               :: dx
!
      print *
      print *
      print *,"trapsimp"
      print *

      print *," input lower, upper limits of x, and number of points of integration (a,b,n)"

      read (unit=*,fmt=*) a,b,n
      print *
      print *,"lower limit   ", a
      print *,"upper limit   ", b
      print *,"number of pts ",n 
      print *
!
      allocate(y(n),stat=ios)
!
      if( ios > 0)then
        print *,"allocation error y"
        stop
      endif

      do k=1,2
        ans = 0.0
        dx  = (b-a)/(n-1)

        do i=1,n
          y(i) = f(a+(i-1)*dx)
        enddo
!
        smyie=0
!
        do i=2,n-1,2
          smyie = smyie+y(i)
        enddo
!
        smyio=0
!
        do i=3,n-2,2
          smyio=smyio+y(i)
        enddo
!
        ans=(y(1)+2*k*smyie+2*smyio+y(n))*dx/(k+1)
!
        select case(k)
          case(1)
            print "(a,f12.8)"," trapezoidal result =", ans
          case(2)
            print "(a,f12.8)"," simpson     result =", ans
          case default
            print *,"improper case k=",k
            stop
        endselect 

      enddo
!
      print "(a,f12.8)"," exact       result =", 3.141593

      print *
      print *
 
      deallocate(y)

      endprogram trapsimp

      function f(x) result(f_result)
      real, intent(in) :: x 
      real             :: f_result
      !f_result=sqrt(1.0-(x-1)*(x-1))
      f_result=sqrt(4.0-x*x)
      return
      end

